<?php
namespace OrganizationsManagement\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\Framework\Controller\Simple;
use OrganizationsManagement\Service;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;

class Organizacao extends Simple {

    private $serviceOrganizacao;

    public function __construct(Service\Organizacao $organizationService){
        $this->serviceOrganizacao = $organizationService;
    }

    public function obtemTodos(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $isPrimary =  Utilities::getValueInFilterDataSourceByName($filters, 'IS_PRIMARY', true);
            $organizations = $this->serviceOrganizacao->obtemTodos($filters, $page, $pageSize, $orderBy, null, $isPrimary);
            $response->addDataSet(new DataSource\DataSet('allOrganizations', $organizations));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function saveOrganizationRelations(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $requestRow = $request->getDataSet()->getRows()[0];
            $nrOrgPrincipal = $requestRow["NRORGPRINCIPAL"];
            $nrOrg = $requestRow["NRORG"];
            $this->serviceOrganizacao->saveOrganizationRelations($nrOrgPrincipal,$nrOrg);
            $response->addMessage(new DTO\Response\Message(Messages::SUC_SAVED));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function allOrganizationRelations(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $nrorg = Utilities::getValueInFilterDataSourceByName($filters, 'NRORG_PRINCIPAL', true);
            $organizations = $this->serviceOrganizacao->allOrganizationRelations($filters, $nrorg);
            $response->addDataSet(new DataSource\DataSet('allOrganizationRelations', $organizations));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function obtemTodosComElemento(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $organizations = $this->serviceOrganizacao->obtemTodosComElemento($filters, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSource\DataSet('allOrganizationsWithElements', $organizations));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function obtemOrganizacoesComProduto(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $orderBy = $request->getFilterCriteria()->getOrderBy();
            $organizations = $this->serviceOrganizacao->obtemOrganizacoesComProduto($filters, $page, $pageSize, $orderBy);
            $response->addDataSet(new DataSource\DataSet('organizationsWithProduct', $organizations));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function getOrganizations(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters       = $request->getFilterCriteria()->getConditions();
            $page          = $request->getFilterCriteria()->getPage();
            $pageSize      = $request->getFilterCriteria()->getPageSize();
            $orderBy       = $request->getFilterCriteria()->getOrderBy();
            $isPrimary     = Utilities::getValueInFilterDataSourceByName($filters, 'IS_PRIMARY', true);
            $organizations = $this->serviceOrganizacao->obtemTodos($filters, $page, $pageSize, $orderBy, false, $isPrimary);
            $response->addDataSet(new DataSource\DataSet('organizations', $organizations));
        } catch (\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function obtemPorId(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $organizationId = $request->getFilterCriteria()->getConditions()[0]['value'];
            $details = $this->serviceOrganizacao->obtemPorId($organizationId);
            $response->addDataSet(new DataSource\DataSet('details', $details));
        } catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function salva(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $organization = $request->getDataSet()->getRows()[0];
            $saveType = $organization['saveType'];
            if($saveType == 'create'){
                $resposta = $this->cria($organization);
                $response->addDataSet(new DataSource\DataSet('resposta', $resposta));
                $response->addMessage(new DTO\Response\Message(Messages::SUC_SAVED));
            } else if($saveType == 'update') {
                $resposta = $this->atualiza($organization);
                $response->addDataSet(new DataSource\DataSet('resposta', $resposta));
                $response->addMessage(new DTO\Response\Message(Messages::SUC_SAVED));
            } else {
                $response->addMessage(new DTO\Response\Message(Messages::ERR_NOT_SAVED));
            }
        } catch(\Exception $ex){
            $cnpj = $request->getDataSet()->getRows()[0]["NRINSJUREMPR"];
            $verify = $this->serviceOrganizacao->obtemPorCNPJ($cnpj);
            if(isset($verify[0]) && $verify[0]->getIdativo() == 'N'){
                $response->addMessage(new DTO\Response\Message(Messages::ERR_NOT_SAVED_ALREADY));
            }else {
                $response->setError(new Error($ex->getMessage(), $ex->getCode()));
                $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            }
        }
    }

    protected function atualiza($organization){
        try {
            $id = empty($organization['NRORG']) ? null : $organization['NRORG'];
            $companyName = empty($organization['NMRAZSOCIALORG']) ? null : $organization['NMRAZSOCIALORG'];
            $fantasyName = empty($organization['NMORGANIZACAO']) ? null : $organization['NMORGANIZACAO'];
            $type = empty($organization['TIPOORG']) ? null : $organization['TIPOORG'];
            $legalInscriptionNumber = empty($organization['NRINSJUREMPR']) ? null : $organization['NRINSJUREMPR'];
            $isActive = empty($organization['IDATIVO']) ? null : $organization['IDATIVO'];

            $dsemailoper = empty($organization['DSEMAILOPER']) ? null : $organization['DSEMAILOPER'];
            $nmoperador = empty($organization['NMOPERADOR']) ? null : $organization['NMOPERADOR'];

            $streetName = empty($organization['DSLOGRADEMPR']) ? null : $organization['DSLOGRADEMPR'];
            $number = empty($organization['NRLOGRADEMPR']) ? null : $organization['NRLOGRADEMPR'];
            $complement = empty($organization['DSCMPLOGEMPR']) ? null : $organization['DSCMPLOGEMPR'];
            $cep = empty($organization['NRCEPEMPR']) ? null : $organization['NRCEPEMPR'];
            $countryId = empty($organization['CDPAIS']) ? null : $organization['CDPAIS'];
            $stateSymbol = empty($organization['SGESTADO']) ? null : $organization['SGESTADO'];
            $cityId = empty($organization['CDMUNICIPIO']) ? null : $organization['CDMUNICIPIO'];
            $neighborhood = empty($organization['SA_BAIRRO']) ? null : $organization['SA_BAIRRO'];

            $cityInscription = empty($organization['CDINSCMUNI']) ? null : $organization['CDINSCMUNI'];
            $stateInscription = empty($organization['NRINSCESTA']) ? null : $organization['NRINSCESTA'];
            $nrParcNegocio = empty($organization['NRPARCNEGOCIO']) ? null : $organization['NRPARCNEGOCIO'];
            $idOrgSaas =  empty($organization['IDORGSAAS']) ? null : $organization['IDORGSAAS'];
            $prepStatus =  empty($organization['PREPSTATUS']) ? null : $organization['PREPSTATUS'];
            $nrOrgPrincipal =  empty($organization['NRORGPRINCIPAL']) ? null : $organization['NRORGPRINCIPAL'];
            $isPrimary = empty($organization['IS_PRIMARY']) ? null : $organization['IS_PRIMARY'];

            $organizationsByLegalInscriptionNumber = $this->serviceOrganizacao->obtemPorCNPJ($legalInscriptionNumber);
            if(!empty($organizationsByLegalInscriptionNumber)) {
                foreach ($organizationsByLegalInscriptionNumber as $organizationByLegalInscriptionNumber) {
                    if ($organizationByLegalInscriptionNumber->getNrinscricaoorg() == $organization["NRINSCRICAOORG"]) {
                        $this->serviceOrganizacao->atualiza(
                            $id, $companyName, $fantasyName, $type, $legalInscriptionNumber, $isActive,
                            $dsemailoper, $nmoperador,
                            $streetName, $number, $complement, $cep, $countryId, $stateSymbol, $cityId, $neighborhood,
                            $cityInscription, $stateInscription, $nrParcNegocio, $idOrgSaas, $prepStatus, $nrOrgPrincipal, $isPrimary
                        );
                        break;
                    }else{
                        throw new \Exception(Messages::ERR_NOT_SAVED_ALREADY_ACTIVE);
                    }
                }
            }else {
                $this->serviceOrganizacao->atualiza(
                    $id, $companyName, $fantasyName, $type, $legalInscriptionNumber, $isActive,
                    $dsemailoper, $nmoperador,
                    $streetName, $number, $complement, $cep, $countryId, $stateSymbol, $cityId, $neighborhood,
                    $cityInscription, $stateInscription, $nrParcNegocio, $idOrgSaas, $prepStatus, $nrOrgPrincipal, $isPrimary
                );
            }

            return array('id' => $id);
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    protected function cria($organization){
        try {
            $companyName = empty($organization['NMRAZSOCIALORG']) ? null : $organization['NMRAZSOCIALORG'];
            $fantasyName = empty($organization['NMORGANIZACAO']) ? null : $organization['NMORGANIZACAO'];
            $type = empty($organization['TIPOORG']) ? null : $organization['TIPOORG'];
            $legalInscriptionNumber = empty($organization['NRINSJUREMPR']) ? null : $organization['NRINSJUREMPR'];
            $isActive = empty($organization['IDATIVO']) ? null : $organization['IDATIVO'];

            $dsemailoper = empty($organization['DSEMAILOPER']) ? null : $organization['DSEMAILOPER'];
            $nmoperador = empty($organization['NMOPERADOR']) ? null : $organization['NMOPERADOR'];

            $streetName = empty($organization['DSLOGRADEMPR']) ? null : $organization['DSLOGRADEMPR'];
            $number = empty($organization['NRLOGRADEMPR']) ? null : $organization['NRLOGRADEMPR'];
            $complement = empty($organization['DSCMPLOGEMPR']) ? null : $organization['DSCMPLOGEMPR'];
            $cep = empty($organization['NRCEPEMPR']) ? null : $organization['NRCEPEMPR'];
            $countryId = empty($organization['CDPAIS']) ? null : $organization['CDPAIS'];
            $stateSymbol = empty($organization['SGESTADO']) ? null : $organization['SGESTADO'];
            $cityId = empty($organization['CDMUNICIPIO']) ? null : $organization['CDMUNICIPIO'];
            $neighborhood = empty($organization['SA_BAIRRO']) ? null : $organization['SA_BAIRRO'];

            $cityInscription = empty($organization['CDINSCMUNI']) ? null : $organization['CDINSCMUNI'];
            $stateInscription = empty($organization['NRINSCESTA']) ? null : $organization['NRINSCESTA'];
            $nrParcNegocio = empty($organization['NRPARCNEGOCIO']) ? null : $organization['NRPARCNEGOCIO'];
            $idOrgSaas =  empty($organization['IDORGSAAS']) ? null : $organization['IDORGSAAS'];
            $nrOrgPrincipal =  empty($organization['NRORGPRINCIPAL']) ? null : $organization['NRORGPRINCIPAL'];

            $id = $this->serviceOrganizacao->cria(
                $companyName, $fantasyName, $type, $legalInscriptionNumber, $isActive,
                $dsemailoper, $nmoperador,
                $streetName, $number, $complement, $cep, $countryId, $stateSymbol, $cityId, $neighborhood,
                $cityInscription, $stateInscription, $nrParcNegocio, $idOrgSaas, $nrOrgPrincipal
            );
            
            return array('id' => $id);
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    public function removeVarios(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $organizations = $request->getDataSet()->getRows();
            $this->serviceOrganizacao->removeVarios($organizations);
            $response->addMessage(new DTO\Response\Message(Messages::SUC_REMOVED));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($this->mensagemLegivel($ex->getMessage())));
        }
    }

    public function removePorId(DTO\Request\Row $requisicao, DTO\Response $resposta){
        try {
            $organizacao = $requisicao->getRow();
            $idOrganizacao = $organizacao['NRORG'];
            $this->serviceOrganizacao->removePorId($idOrganizacao);
            $resposta->addMessage(new DTO\Response\Message(Messages::SUC_REMOVED));
        } catch(\Exception $ex){
            $resposta->addMessage(new DTO\Response\Message($this->mensagemLegivel($ex->getMessage())));
        }
    }

    private function mensagemLegivel($message){
        $translatedMessage = $message;
        $errorMessages = array(
            "integrity" => Messages::ERR_ORG_NOT_REMOVED_DUE_CONSTRAINT,
            "integridade" => Messages::ERR_ORG_NOT_REMOVED_DUE_CONSTRAINT
        );
        foreach($errorMessages as $substring => $translated){
            if(strpos($message, $substring)){
                $translatedMessage = $translated;
                break;
            }
        }
        return $translatedMessage;
    }
    
    public function activateNotSaas(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $information = $request->getDataSet()->getRows()[0];
            $id = $information['NRORG'];
            $nmoperador = $information['NMOPERADOR'];
            $dsemailoper = $information['DSEMAILOPER'];
            $this->serviceOrganizacao->activateNotSaas($id, $nmoperador, $dsemailoper);
        } catch (Exception $ex) {
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function prepara(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $information = $request->getDataSet()->getRows()[0];
            $id = $information['NRORG'];
            $this->serviceOrganizacao->prepara($id);
            $response->addMessage(new DTO\Response\Message("Organização preparada com sucesso."));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function solicitaPreparacao(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $informacoes = $req->getDataSet()->getRows()[0];
            $email = $informacoes['EMAIL'];
            $idOrganizacao = $informacoes['NRORG'];

            $this->serviceOrganizacao->solicitaPreparacao($idOrganizacao, $email);
            $res->addMessage(new DTO\Response\Message("Solicitação enviada com sucesso."));
        } catch(\Exception $ex){
            $res->addMessage(new DTO\Response\Message($ex->getMessage()));
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function changeEnvironment(DTO\Request\DataSet $req, DTO\Response $res) {
        try {
            $row           = $req->getDataSet()->getRows()[0];
            $idOrgSaas     = $row['IDORGSAAS'];
            $idOrganizacao = $row['NRORG'];
            $this->serviceOrganizacao->changeEnvironment($idOrganizacao, $idOrgSaas);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function changeStatusBkp(DTO\Request\DataSet $req, DTO\Response $res) {
        try {
            $row           = $req->getDataSet()->getRows()[0];
            $prepStatusBkp = $row['PREPSTATUSBKP'];
            $idOrganizacao = $row['NRORG'];
            $this->serviceOrganizacao->changeStatusBkp($idOrganizacao, $prepStatusBkp);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function requestDeactivation(DTO\Request\DataSet $req, DTO\Response $res) {
        try {
            $informacoes = $req->getDataSet()->getRows()[0];
            $email = $informacoes['EMAIL'];
            $idOrganizacao = $informacoes['NRORG'];
            $this->serviceOrganizacao->requestDeactivation($idOrganizacao, $email);
            $res->addMessage(new DTO\Response\Message(Messages::SUC_REQUEST_DEACTIVATION_ORGANIZATION));
        } catch(\Exception $ex){
            $res->addMessage(new DTO\Response\Message($ex->getMessage()));
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function habilitaPreparacao(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $id = $req->getDataSet()->getRows()[0]['NRORG'];
            $this->serviceOrganizacao->habilitaPreparacao($id);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getStructureCommands(DTO\Request\Filter $req, DTO\Response $res){
        try {
            $filters     = $req->getFilterCriteria()->getConditions();
            $action      = Utilities::getValueInFilterDataSourceByName($filters, 'IDACAOEXECOM');
            $environment = Utilities::getValueInFilterDataSourceByName($filters, 'IDAMBCOMANDO');
            $commands    = $this->serviceOrganizacao->getStructureCommands($environment, $action);
            $res->addDataSet(new DataSource\DataSet('commands', $commands));
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function execStructureCommand(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $row         = $req->getDataSet()->getRows()[0];
            $command     = $row['COMANDO'];
            $url         = $row['URL'];
            $commandType = $row['IDTPCOMANDO'];
            $environment = $row['IDAMBCOMANDO'];
            $orgId       = $row['ORGANIZATION_ID'];
            $result      = $this->serviceOrganizacao->execStructureCommand($command, $commandType, $url, $environment, $orgId);
            $res->addDataSet(new DataSource\DataSet('result', $result));
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function setStatusInactive(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusInactive($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function setStatusBlocked(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusBlocked($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function setStatusBackup(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusBackup($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function setStatusStructureRemoved(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusStructureRemoved($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function setStatusGenerateStructure(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusGenerateStructure($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function setStatusPreparedZmart(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusPreparedZmart($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function setStatusPrepared(DTO\Request\DataSet $req, DTO\Response $res){
        try {
            $valores = $req->getDataSet()->getRows()[0];
            $idOrganizacao = $valores['NRORG'];
            $this->serviceOrganizacao->setStatusPrepared($idOrganizacao);
        } catch(\Exception $ex){
            $res->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getOrganizationRestrictionByIp(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters      = $request->getFilterCriteria()->getConditions();
            $orgId        = Utilities::getValueInFilterDataSourceByName($filters, 'NRORG', true);
            $restrictions = $this->serviceOrganizacao->getOrganizationRestrictionByIp($orgId, $filters);
            $response->addDataSet(new DataSource\DataSet('organizationRestrictionByIp', $restrictions));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }
    
    public function saveOrganizationRestrictionByIp(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $ipv4 = $row['IPV4'];
            $ipv6 = $row['IPV6'];
            $orgId = $row['ORG_ID'];
            $idIpRestricted = $row['ID'];
            $active = $row['ACTIVE'];
            $description = $row['DESCRIPTION'];
            $restriction = $this->serviceOrganizacao->saveOrganizationRestrictionByIp($idIpRestricted, $ipv4, $ipv6, $orgId, $description, $active);
            $response->addMessage(new DTO\Response\Message(Messages::SUC_SAVED));
            $response->addDataSet(new DataSource\DataSet('organizationRestrictionByIp', $restriction));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function removeOrganizationRestrictionByIp(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            $restrictionId = $row['ID'];
            $orgId = $row['ORG_ID'];
            $this->serviceOrganizacao->removeOrganizationRestrictionByIp($restrictionId, $orgId);
            $response->addMessage(new DTO\Response\Message(Messages::SUC_REMOVED));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getOrganizationParameters(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = $this->serviceOrganizacao->getOrganizationParameters($filters);
            $response->addDataSet(new DataSource\DataSet('parameters', $parameters));
        } catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function saveOrganizationParameters(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $orgId            = $row['ORG_ID'];
            $useRestriction   = empty($row['USE_RESTRICTION'])    ? 'N'  : $row['USE_RESTRICTION'];
            $allowProductCall = empty($row['IDALLOWPRODUCTCALL']) ? 'N'  : $row['IDALLOWPRODUCTCALL'];
            $idColetor        = empty($row['IDCOLETOR'])          ? null : $row['IDCOLETOR'];
            $dsStringConexao  = empty($row['DSSTRINGCONEXAO'])    ? null : $row['DSSTRINGCONEXAO'];
            $idUtWallet       = empty($row['IDUTWALLET'])         ? null : $row['IDUTWALLET'];
            $dsUsuarioBanco   = empty($row['DSUSUARIOBANCO'])     ? null : $row['DSUSUARIOBANCO'];
            $dsSenhaBanco     = empty($row['DSSENHABANCO'])       ? null : $row['DSSENHABANCO'];
            $idTpBanco        = empty($row['IDTPBANCO'])          ? null : $row['IDTPBANCO'];
            $saveParametersResponse = $this->serviceOrganizacao->saveOrganizationParameters($orgId, $useRestriction, $allowProductCall, 
                $idColetor, $dsStringConexao, $idUtWallet, $dsUsuarioBanco, $dsSenhaBanco, $idTpBanco);
            $response->addDataSet(new DataSource\DataSet('saveParametersResponse', $saveParametersResponse));
            if(!$saveParametersResponse['NOT_DB_USER']) {
                $response->addMessage(new DTO\Response\Message(Messages::SUC_SAVED));
            }
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function getDefaultParameters(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = $this->serviceOrganizacao->getDefaultParameters($filters);
            $response->addDataSet(new DataSource\DataSet('defaultParameters', $parameters));
        } catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function saveDefaultParameters(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row             = $request->getDataSet()->getRows()[0];
            $id              = $row['ID'];
            $idativo         = $row['IDATIVO'];
            $idabmorg        = $row['IDAMBORG'];
            $idColetor       = empty($row['IDCOLETOR'])       ? null : $row['IDCOLETOR'];
            $dsStringConexao = empty($row['DSSTRINGCONEXAO']) ? null : $row['DSSTRINGCONEXAO'];
            $idUtWallet      = empty($row['IDUTWALLET'])      ? null : $row['IDUTWALLET'];
            $dsUsuarioBanco  = empty($row['DSUSUARIOBANCO'])  ? null : $row['DSUSUARIOBANCO'];
            $dsSenhaBanco    = empty($row['DSSENHABANCO'])    ? null : $row['DSSENHABANCO'];
            $idTpBanco       = empty($row['IDTPBANCO'])       ? 'ORACLE' : $row['IDTPBANCO'];
            $saveParameters  = $this->serviceOrganizacao->saveDefaultParameters($id,  $idColetor,
                $dsStringConexao, $idUtWallet, $dsUsuarioBanco, $dsSenhaBanco, $idativo, $idabmorg, $idTpBanco);
            $response->addDataSet(new DataSource\DataSet('saveParameters', $saveParameters));
        } catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function removeDefaultParameters(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $id  = $row['ID'];
            $this->serviceOrganizacao->removeDefaultParameters($id);
        } catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function generateLoginAccessFile(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $orgId = $row['ORGANIZATION_ID'];
            $productsWithoutElements = $this->serviceOrganizacao->generateLoginAccessFile($orgId);
            $response->addDataSet(new DataSource\DataSet('productsWithoutElements', $productsWithoutElements));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function downloadLoginAccessFile(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $orgId = $row['ORGANIZATION_ID'];
            $this->serviceOrganizacao->downloadLoginAccessFile($orgId);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function downloadStructureExportation(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $orgId = $row['ORGANIZATION_ID'];
            $this->serviceOrganizacao->downloadStructureExportation($orgId);
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
        }
    }

    public function saveOrganizationValidation(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters = $request->getFilterCriteria()->getConditions();
            $nrIncricao = Utilities::getValueInFilterDataSourceByName($filters, 'NRINSCRICAO');
            $tipoOrg = Utilities::getValueInFilterDataSourceByName($filters, 'TIPOORG');
            $clienteParcNegocio = $this->serviceOrganizacao->saveOrganizationValidation($nrIncricao, $tipoOrg);
            $response->addDataSet(new DataSource\DataSet('saveOrgValidation', $clienteParcNegocio));
        } catch(\Exception $ex){
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function saveOrganizationAppCode(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $requestRow = $request->getDataSet()->getRows()[0];
            $cdappnotif = $requestRow['CDAPPNOTIF'];
            $dschavenotif = $requestRow['DSCHAVENOTIF'];
            $productOrgId = $requestRow['PRODUCT_ORG'];
            $idProdSaas = $requestRow['IDPRODUTOSAAS'];
            $urlRedirect = $requestRow['DSURLREDIRECT'];
            $urlAcesso = $requestRow['DSURLACESSO'];
            $idProdutoExclusivo = $requestRow["IDPRODUTOEXCLUSIVO"];
            $organization = InstanceProvider::getService()->getOrganizacao();
            $dateSet = $organization->saveOrganizationAppCode($dschavenotif, $cdappnotif, $productOrgId, $idProdSaas, $urlRedirect, $idProdutoExclusivo, $urlAcesso);
            $response->addDataSet(new DataSet('product', $dateSet));
        } catch (Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
        }
    }
    
    public function getProductOrg(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $serviceOrganizacao = InstanceProvider::getService()->getOrganizacao();
            $dataSet = $serviceOrganizacao->getProductOrg($conditions);
            $response->addDataSet(new DataSource\DataSet('productOrg', $dataSet));
        } catch (Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
        }
    }

    public function saveOrganizationLog(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row   = $request->getDataSet()->getRows()[0];
            $log   = $row['LOG'][0]['path'];
            $orgId = $row['ORG_ID'];
            InstanceProvider::getService()->getOrganizacao()->saveOrganizationLog($log, $orgId);
        } catch (Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
        }
    }

    public function verifyBaseErrors(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $orgId      = Utilities::getValueInFilterDataSourceByName($conditions, 'NRORG');

            $result = InstanceProvider::getService()->getOrganizacao()->verifyBaseErrors($orgId);
            $response->addDataSet(new DataSource\DataSet('errors', [$result]));
        } catch (Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
        }
    }

    public function getOrganizationLog(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $orgId      = Utilities::getValueInFilterDataSourceByName($conditions, 'ORG_ID');
            $log        = InstanceProvider::getService()->getOrganizacao()->getOrganizationLog($orgId);
            $response->addDataSet(new DataSource\DataSet('log', [$log]));
        } catch (Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), 500));
        }
    }

    public function downloadOrganizationLog(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row   = $request->getDataSet()->getRows()[0];
            $orgId = $row['ORGANIZATION_ID'];
            $this->serviceOrganizacao->downloadOrganizationLog($orgId);
        } catch(\Exception $ex){
            $response->setError(new Error($ex->getMessage(), $ex->getCode()));
        }
    }
}
