<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\Messages as Messages;
use OrganizationsManagement\Factory;
use Teknisa\Model\Teknisa\Entities;
use Teknisa\Model\Geral\Entities\Endeclie;
use Teknisa\Model\Geral\Entities\Cliente;
use Teknisa\Model\Geral\Entities\Organizacao as OrganizacaoEntity;
use Teknisa\Model\Geral\Repositories as GeralRepositories;
use Teknisa\Model\Teknisa\Repositories as TeknisaRepositories;
use Teknisa\Libs\Util\Environment as TeknisaEnvironment;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Mail\SmtpCore;

class Api
{
    /** @var Utilities * */
    protected $utilities;
    /** @var EntityManager * */
    protected $entityManager;
    /** @var TeknisaEnvironment * */
    protected $environment;

    /** @var GeralRepositories\Organizacao */
    private $organizacaoRepository;

    /** @var GeralRepositories\Organizacao */
    private $clienteRepository;

    /** @var GeralRepositories\Enderecoparc */
    private $enderecoparcRepository;

    /** @var TeknisaRepositories\OmProdutoOrg */
    private $produtoOrgRepository;

    /** @var TeknisaRepositories\OmProduto */
    private $productRepository;

    /** @var TeknisaRepositories\OmProdutoElemento * */
    protected $productElementRepository;

    /** @var TeknisaRepositories\OmProdutoOrgElemento * */
    protected $productOrgElementRepository;

    /** @var Service/Organizacao */
    protected $organizationService;

    /** @var Service/OrgElemento */
    protected $orgElementService;

    /** @var Service/Organizacao */
    protected $productService;

    /** @var Factory\ProdutoOrg * */
    protected $produtoOrgFactory;

    protected $allConnections = array();

    const VARIABLE_MEASURE_BASE_SUBJECT_EMAIL = 'Erro no envio da coleta de Medida Variável para SM';
    const ERROR_MSG_DISCARDED = 'Valor calculado para a medição de parcela selecionada e medida variável selecionada é igual ou superior ao que está sendo enviado';

    const CUSTOMIZACAO = '1';
    const LICENCAUSO   = '2';
    const MENSALIDADE  = '3';
    const MAODEOBRA    = '5';
    const SUPORTE      = '6';

    public function __construct(EntityManager $entityManager, Utilities $utilities, TeknisaEnvironment $environment,
                                Organizacao $organizationService, Produto $productService, OrgElemento $orgElementService,
                                Factory\ProdutoOrg $produtoOrgFactory)
    {
        $this->utilities = $utilities;
        $this->environment = $environment;
        $this->entityManager = $entityManager;

        $this->produtoOrgFactory = $produtoOrgFactory;

        $this->productService = $productService;
        $this->orgElementService = $orgElementService;
        $this->organizationService = $organizationService;

        $this->clienteRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Cliente');
        $this->endeclieRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Endeclie');
        $this->productRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
        $this->organizacaoRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Organizacao');
        $this->enderecoparcRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Enderecoparc');
        $this->produtoOrgRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrg');
        $this->productElementRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoElemento');
        $this->productOrgElementRepository = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrgElemento');
    }

    /**
     * @param Cliente $customer
     * @return null|object
     */
    private function checkIfHasOrganizationWithSameDataCustomer(Cliente $customer)
    {
        $organization = $this->organizacaoRepository->findOneBy(array('nrparcnegocio' => $customer->getNrparcnegocio()));
        if ($organization) {
            return $organization;
        }
    }

    /**
     * @param $requesterOrganizationId
     * @param $requesterUserId
     * @param $clientId
     * @param $operatorName
     * @param $operatorEmail
     * @param $products
     * @param $expirationDate
     * @param $isExtendContract
     * @param $contractCode
     * @param $initialDate
     * @throws \Exception
     * @return array
     */
    public function saveOrganizationWithProductsAndElementsRules($requesterOrganizationId, $requesterUserId, $clientId, $operatorName, $operatorEmail,
                                                                 $products, $expirationDate, $isExtendContract, $contractCode, $initialDate, $oldContractCode)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        array_push($this->allConnections, $connection);
        try {
            $dateNow = new \DateTime();
            /** @var Cliente $customer */
            $customer = $this->clienteRepository->findOneBy(array("cdcliente" => $clientId));
            if (!$customer) {
                throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_CUSTOMER_NOT_FOUND);
            }

            $customerMainAddress = $this->endeclieRepository->findOneBy(array("cdcliente" => $customer->getCdcliente(), "idtpendeclie" => Endeclie::CUSTOMER_MAIN_ADDRESS));
            if (!$customerMainAddress) {
                throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_CUSTOMER_NOT_HAS_MAIN_ADDRESS);
            }

            $customerDetailsMainAddress = $this->enderecoparcRepository->findOneBy(array("nrenderecoparc" => $customerMainAddress->getNrenderecoparc(), "nrorg" => $customerMainAddress->getNrorg()));

            /** @var OrganizacaoEntity $organizationEntity */
            $organizationEntity = $this->checkIfHasOrganizationWithSameDataCustomer($customer);
            if (!$organizationEntity) {
                $nmoperador = $operatorName;
                $dsemailoper = $operatorEmail;
                $idativo = $customer->getIdativo();
                $nmorganizacao = $customer->getNmfantclie();
                $tipoorg = $customer->getIdtpijurclie();
                $nmrazsocialorg = $customer->getNmrazsocclie();
                $nrinsjurempr = $customer->getNrinsjurclie();
                $nrinscesta = $customer->getNrinsestclie();
                $cdinscmuni = $customer->getNrinsmuniclie();
                $nrParcNegocio = $customer->getNrparcnegocio();
                $cdpais = $customerDetailsMainAddress->getCdpais();
                $sgestado = $customerDetailsMainAddress->getSgestado();
                $dslogradempr = $customerDetailsMainAddress->getDsendereco();
                $cdmunicipio = $customerDetailsMainAddress->getCdmunicipio();
                $nrcepempr = $customerDetailsMainAddress->getNrcependereco();
                $nrlogradempr = $customerDetailsMainAddress->getNrimovelendereco();
                $dscmplogempr = $customerDetailsMainAddress->getDscompleendereco();
                $nmbairro = $customerDetailsMainAddress->getNmbairroendereco();

                $organizationEntity = $this->organizationService->saveOrganization($nmrazsocialorg, $nmorganizacao, $tipoorg,
                    $nrinsjurempr, $idativo, $dsemailoper, $nmoperador, $dslogradempr, $nrlogradempr, $dscmplogempr, $nrcepempr,
                    $cdpais, $sgestado, $cdmunicipio, $nmbairro, $cdinscmuni, $nrinscesta, $nrParcNegocio);
                $this->entityManager->persist($organizationEntity);
            }

            $this->saveProductsAndElements($products, $organizationEntity, $requesterUserId, $requesterOrganizationId,
                $dateNow, $expirationDate, $isExtendContract, $clientId, $contractCode, $initialDate, $oldContractCode);

            /** @var Connection $connectionManagement */
            foreach (array_reverse($this->allConnections) as $connectionManagement) {
                $connectionManagement->commit();
            }

            return array("organizationId" => $organizationEntity->getNrorg());
        } catch (\Exception $e) {
            foreach (array_reverse($this->allConnections) as $connectionManagement) {
                $connectionManagement->rollBack();
            }
            throw new \Exception($e->getMessage());
        }
    }

    public function removeOrganizationProductElements($nrorgId, $products){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $elementsNotFound = array();
            $nrprodutoOrgId = null;
            $nrprodutoElementoId = null;
            foreach ($products as $product){
                $productId = $product['NRPRODUTO'];
                foreach ($product['ELEMENTOS'] as $element){
                    $nrprodutoOrg = $this->produtoOrgRepository->findOneBy(array(
                        "nrorgId" => $nrorgId,
                        "nrprodutoId" => $productId
                    ));
                    if(!$nrprodutoOrg){
                        throw new \Exception(Messages::ERR_NO_PRODUCT_ORGANIZATION_RELATION);
                    }
                    $nrprodutoOrgId = $nrprodutoOrg->getId();
                    $nrprodutoElemento = $this->productElementRepository->findOneBy(array(
                        "uniqueId" => $element,
                        "nrprodutoId" => $productId
                    ));
                    if(!$nrprodutoElemento){
                        $elementToRemove = null;
                        array_push($elementsNotFound, $element);
                    } else{
                        $nrprodutoElementoId = $nrprodutoElemento->getId();
                        $elementToRemove = $this->productOrgElementRepository->findOneBy(array(
                            "nrprodutoOrgId" => $nrprodutoOrgId,
                            "nrprodutoElementoId" => $nrprodutoElementoId
                        ));
                    }
                    if($elementToRemove){
                        $this->entityManager->remove($elementToRemove);
                        $this->entityManager->flush();
                    }
                }
                $produtoOrgElemento = $this->productOrgElementRepository->findOneBy(array(
                   "nrprodutoOrgId" => $nrprodutoOrgId,
                ));
                if(!$produtoOrgElemento){
                    $orgToRemove = $this->produtoOrgRepository->findOneBy(array(
                        "nrorgId" => $nrorgId,
                        "nrprodutoId" => $productId
                    ));
                    $this->entityManager->remove($orgToRemove);
                    $this->entityManager->flush();
                }
            }
            $connection->commit();
            return $elementsNotFound;
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function validateOrganizationByElements($products)
    {
        $irregularProducts = null;
        foreach ($products as $count => $product) {
            $productDetails = $this->productRepository->findOneBy(array('id' => $product['productId']));
            if (!empty($product['productId'])) {
                $elements = $product['elements'];
                foreach ($elements as $i => $element) {
                    $elementEntity = $this->productElementRepository->findOneBy(array('uniqueId' => $element['id'], 'nrprodutoId' => $product['productId']));
                    if ($elementEntity) {
                        $result = array(
                            "ValidatedElements" => true
                        );
                    } else {
                        $irregularElements[$i] = array(
                            'element' => array($element['id'])
                        );
                        $irregularProducts[$count] = array(
                            "product" => $productDetails->getNmproduto(),
                            "element" => $irregularElements,
                            "ValidatedElements" => false
                        );
                    }
                }
            } else {
                throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_PRODUCT_ID_NOT_INFORMED);
            }
        }
        if ($irregularProducts) {
            return $irregularProducts;
        }
        return array($result);
    }

    private function saveProductsAndElements($products, $organizationEntity, $requesterUserId, $requesterOrganizationId, $dateNow,
                                             $expirationDate, $isExtendContract, $clientId, $contractCode, $initialDate, $oldContractCode)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        array_push($this->allConnections, $connection);

        foreach ($products as $product) {
            if (!empty($product['productId'])) {
                /** @var Entities\OmProduto $productDetails */
                $productDetails = $this->productRepository->findOneBy(array('id' => $product['productId']));
                if ($productDetails) {
                    $productOrg = $this->saveProductToOrganization($organizationEntity->getNrorg(), $product['productId'],
                        $requesterUserId, $requesterOrganizationId, $dateNow);
                    $this->entityManager->persist($productOrg);
                    $this->entityManager->flush();
                    $this->saveElementsToOrganization($requesterOrganizationId, $requesterUserId, $product['elements'],
                        $productOrg, $productDetails, $expirationDate, $isExtendContract, $contractCode);
                    $isCreateProductLine = isset($product['isCreateProductLine']) ? $product['isCreateProductLine'] : true;
                    if($isCreateProductLine) {
                        $vrlicencusolinp = isset($product['licenseValue'])       ? $product['licenseValue']       : null;
                        $vrmensalidalinp = isset($product['licenseMaintenance']) ? $product['licenseMaintenance'] : null;
                        $qtMinLicenses   = isset($product['qtMinLicenses'])      ? $product['qtMinLicenses']      : null;
                        $contractualName = isset($product['contractualName'])    ? $product['contractualName']    : null;
                        $this->saveProductLine($productDetails, $productOrg, $requesterUserId, $requesterOrganizationId, $dateNow, $clientId, $contractCode, $initialDate, 
                            $expirationDate, $vrlicencusolinp, $vrmensalidalinp, $qtMinLicenses, $contractualName, $isExtendContract, $product['elements'], $oldContractCode);
                    }
                } else {
                    throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_PRODUCT_NOT_FOUND_EXPLANATION . $product['productId']);
                }
            } else {
                throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_PRODUCT_ID_NOT_INFORMED);
            }
        }
    }

    private function productLineParametersValidation($elements, $nmProduto, $contractCode, $initialDate, $expirationDate, $qtMinLicenses)
    {
        if (empty($elements)) {
            $message = Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_SYSTEM_PRODUCT_LINE_NOT_FOUND;
            throw new \Exception(str_replace("{{productName}}", $nmProduto, $message));
        } else {
            $emptyParameters = array();
            if (empty($contractCode)) {
                $emptyParameters[] = '"Código do contrato"';
            }
            if (empty($initialDate)) {
                $emptyParameters[] = '"Data inicial do contrato"';
            }
            if (empty($expirationDate)) {
                $emptyParameters[] = '"Data final do contrato"';
            }
            if (empty($qtMinLicenses) && strval($qtMinLicenses) !== '0') {
                $emptyParameters[] = '"Quantidade mínima de licenças do produto ' . $nmProduto . '"';
            }
            if (!empty($emptyParameters)) {
                throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_PRODUCT_LINE_PARAMS_NOT_FOUND . join(', ', $emptyParameters));
            }
        }
    }

    /**
     * @param Entities\OmProduto $productDetails
     * @param Entities\OmProdutoOrg $productOrg
     * @param $requesterUserId
     * @param $requesterOrganizationId
     * @param $dateNow
     * @param $clientId
     * @param $contractCode
     * @param $initialDate
     * @param $expirationDate
     * @param $vrlicencusolinp
     * @param $vrmensalidalinp
     * @param $qtMinLicenses
     * @param $contractualName
     * @param $isExtendContract
     * @param $smElements
     * @throws \Exception
     */
    private function saveProductLine($productDetails, $productOrg, $requesterUserId, $requesterOrganizationId, $dateNow, $clientId, $contractCode, $initialDate,
                                     $expirationDate, $vrlicencusolinp, $vrmensalidalinp, $qtMinLicenses, $contractualName, $isExtendContract, $smElements, $oldContractCode)
    {
        $dsLinhaProd   = $productDetails->getDslinhaproduto();
        $idmodoproc    = $productDetails->getIdmodoproc();
        $idtplinhaprod = $productDetails->getIdtplinhaproduto();
        $dsLinhaProd   = empty($contractualName) ? $dsLinhaProd : $contractualName;
        if ($dsLinhaProd && $idmodoproc && $idtplinhaprod) {
            $cdlinpclie   = $productOrg->getId() . '_' . $contractCode;
            $nrproposfas  = Utilities::getDefaultNrproposfasParameter();
            $nrverpropfas = Utilities::getDefaultNrverpropfasParameter();

            if (empty($vrlicencusolinp)) $vrlicencusolinp = 0;
            if (empty($vrmensalidalinp)) $vrmensalidalinp = 0;

            $nrusualinpc   = $productDetails->getCdusutpmedidaserv() ? $qtMinLicenses : '999';
            $qtMinLicenses = $productDetails->getCdusutpmedidaserv() ? '0' : $qtMinLicenses;

            /** @var Entities\Contratolinp $productLine */
            $productLine = InstanceProvider::getRepository()->getContratolinp()->findOneBy(array('cdcliente' => $clientId,
                'nrseqcontrato' => $contractCode, 'nrprodutoId' => $productDetails->getId()));
            if(empty($productLine) && $oldContractCode) {
                $productLine = InstanceProvider::getRepository()->getContratolinp()->findOneBy(array('cdcliente' => $clientId,
                    'nrseqcontrato' => $oldContractCode, 'nrprodutoId' => $productDetails->getId()));
            }

            $smElements = array_filter($smElements, function($element) {
                return isset($element['isUpdate']) ? $element['isUpdate'] : true;
            });
            $smElements = Utilities::getArrayFromAssociativeArray($smElements, 'id');
            $smElementsFilter = " AND (" . Utilities::fixBugWhereINQuery($smElements, 'UNIQUE_ID', false, true) . ")";
            $elements = $this->productElementRepository->getElementsRelatedSystemByProduct($productDetails->getId(), $smElementsFilter);

            if (empty($productLine)) {
                $this->insertProductLine($elements, $productDetails, $contractCode, $initialDate, $expirationDate,
                    $qtMinLicenses, $dateNow, $requesterOrganizationId, $requesterUserId, $nrproposfas, $nrverpropfas,
                    $clientId, $cdlinpclie, $dsLinhaProd, $idtplinhaprod, $idmodoproc, $vrlicencusolinp, $vrmensalidalinp, $nrusualinpc);
                $this->updateCustomerPER($clientId, $requesterUserId, $requesterOrganizationId, $contractCode);
            } else if ($productLine && ($isExtendContract || $oldContractCode)) {
                $this->updateProductLine($elements, $initialDate, $expirationDate, $qtMinLicenses, $productLine, $dateNow, $requesterOrganizationId,
                    $requesterUserId, $clientId, $productLine->getCdlinpclie(), $dsLinhaProd, $vrlicencusolinp, $vrmensalidalinp, $nrusualinpc, $contractCode, $productDetails->getId());
            }

            $this->entityManager->flush();
        }
    }

    public function updateCustomerPER($clientId, $requesterUserId, $requesterOrganizationId, $contractCode) {
        $parClieFasRegisters = InstanceProvider::getRepository()->getParcliefas()->findOneBy(array('cdcliente' => $clientId, 'idtpfatuclie' => '1'));
        if(empty($parClieFasRegisters)) {
            /** @var Cliente $customer */
            $customer = $this->clienteRepository->findOneBy(array("cdcliente" => $clientId));
            /** @var \Teknisa\Model\Services\Entities\FctContrato $contract */
            $contract = InstanceProvider::getRepository()->getFctContrato()->findOneBy(array('nrseqcontrato' => $contractCode));
            $customer->setIdutliceuso('S');
            $customer->setIdlibgerper('S');
            $customer->setNrdiavalpers(60);
            $customer->setCdagentenegocio($contract->getCdoperresponsavel());
            $operResp = InstanceProvider::getRepository()->getOrganization()->getOperResp($contract->getCdoperresponsavel());
            if (!empty($operResp)) {
                $customer->setCdgerentecomerc($operResp[0]['CDOPERRESP']);
            }

            $parClieFasFactory = InstanceProvider::getFactory()->getParcliefas();
            //Customização
            $dsFaturClie = 'Prestação de serviços de processamento de dados.';
            $dsBoletClie = 'Prestação de serviços de processamento de dados.';
            $parClieFas = $parClieFasFactory->cria($clientId, self::CUSTOMIZACAO, $dsFaturClie, $dsBoletClie, $requesterOrganizationId, $requesterUserId);
            $this->entityManager->persist($parClieFas);
            //Licença de Uso
            $dsFaturClie = 'Cessão de direito de uso do sistema.';
            $dsBoletClie = 'Cessão de direito de uso do sistema.';
            $parClieFas = $parClieFasFactory->cria($clientId, self::LICENCAUSO, $dsFaturClie, $dsBoletClie, $requesterOrganizationId, $requesterUserId);
            $this->entityManager->persist($parClieFas);
            //Manutenção
            $dsFaturClie = 'Manutenção no processamento de dados.';
            $dsBoletClie = 'Manutenção ref a [MM/YYYY].';
            $parClieFas = $parClieFasFactory->cria($clientId, self::MENSALIDADE, $dsFaturClie, $dsBoletClie, $requesterOrganizationId, $requesterUserId);
            $this->entityManager->persist($parClieFas);
            //Mão de Obra
            $dsFaturClie = 'Suporte no processamento de dados ref prop COMxxxxxx-x.';
            $dsBoletClie = 'Prestação de serviços.';
            $parClieFas = $parClieFasFactory->cria($clientId, self::MAODEOBRA, $dsFaturClie, $dsBoletClie, $requesterOrganizationId, $requesterUserId);
            if (!empty($operResp)) {
                $parClieFas->setCdgerenteimpcli($operResp[0]['CDOPERRESP']);
            }
            $this->entityManager->persist($parClieFas);
            //Suporte
            $dsFaturClie = 'Prestação de serviços.';
            $dsBoletClie = 'Prestação de serviços.';
            $parClieFas = $parClieFasFactory->cria($clientId, self::SUPORTE, $dsFaturClie, $dsBoletClie, $requesterOrganizationId, $requesterUserId);
            $this->entityManager->persist($parClieFas);
        }
    }

    /**
     * @param $elements
     * @param Entities\OmProduto $productDetails
     * @param $contractCode
     * @param $initialDate
     * @param $expirationDate
     * @param $qtMinLicenses
     * @param $dateNow
     * @param $requesterOrganizationId
     * @param $requesterUserId
     * @param $nrproposfas
     * @param $nrverpropfas
     * @param $clientId
     * @param $cdlinpclie
     * @param $dsLinhaProd
     * @param $idtplinhaprod
     * @param $idmodoproc
     * @param $vrlicencusolinp
     * @param $vrmensalidalinp
     * @param $nrusualinpc
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    private function insertProductLine($elements, $productDetails, $contractCode, $initialDate, $expirationDate, $qtMinLicenses,
                                       $dateNow, $requesterOrganizationId, $requesterUserId, $nrproposfas, $nrverpropfas, $clientId,
                                       $cdlinpclie, $dsLinhaProd, $idtplinhaprod, $idmodoproc, $vrlicencusolinp, $vrmensalidalinp, $nrusualinpc)
    {
        $linhaProdutoFactory = InstanceProvider::getFactory()->getLinhaProduto();
        $this->productLineParametersValidation($elements, $productDetails->getNmproduto(), $contractCode, $initialDate,
            $expirationDate, $qtMinLicenses);

        $productLine = $linhaProdutoFactory->insertContratoLinp($dateNow, $requesterOrganizationId, $requesterUserId,
            $nrproposfas, $nrverpropfas, $clientId, $cdlinpclie, '01', $dsLinhaProd, $idtplinhaprod,
            $qtMinLicenses, $nrusualinpc, 'WEB', $idmodoproc, $idmodoproc, $productDetails->getId(), $contractCode);
        $this->entityManager->persist($productLine);
        $this->entityManager->flush();

        $linhaProdutoFactory->insertValorlinp($requesterOrganizationId, $requesterUserId,
            $clientId, $cdlinpclie, $initialDate, $expirationDate, $vrlicencusolinp, $vrmensalidalinp);

        foreach ($elements as $element) {
            $systemProductLine = $linhaProdutoFactory->insertSistContLinp($dateNow, $requesterOrganizationId,
                $requesterUserId, $clientId, $cdlinpclie, $element['CDSISTEMA']);
            $this->entityManager->persist($systemProductLine);
        }
    }

    /**
     * @param $elements
     * @param $initialDate
     * @param $expirationDate
     * @param $qtMinLicenses
     * @param Entities\Contratolinp $productLine
     * @param $dateNow
     * @param $requesterOrganizationId
     * @param $requesterUserId
     * @param $clientId
     * @param $cdlinpclie
     * @param $dsLinhaProd
     * @param $vrlicencusolinp
     * @param $vrmensalidalinp
     * @param $nrusualinpc
     * @throws \Doctrine\ORM\ORMException
     */
    private function updateProductLine($elements, $initialDate, $expirationDate, $qtMinLicenses, $productLine,
                                       $dateNow, $requesterOrganizationId, $requesterUserId, $clientId,
                                       $cdlinpclie, $dsLinhaProd, $vrlicencusolinp, $vrmensalidalinp, $nrusualinpc, $contractCode, $productId)
    {
        $linhaProdutoFactory = InstanceProvider::getFactory()->getLinhaProduto();
        $productLine->setNmlinpclie($dsLinhaProd);
        $productLine->setQtminimafatlinp($qtMinLicenses);
        $productLine->setCdoperultatu($requesterUserId);
        $productLine->setNrorgultatu($requesterOrganizationId);
        $productLine->setDtultatu($dateNow);
        $productLine->setNrseqcontrato($contractCode);
        
        //Produtos utilizados para criação de contratos no SM não devem alterar os usuários das suas linhas de produtos (gambiarra pra corrigir uma gambiarra maior).
        if($productId != 4312 && $productId != 4313) {
            $productLine->setNrusualinpc($nrusualinpc);
        }
        
        $this->entityManager->persist($productLine);

        $valorLinp = InstanceProvider::getRepository()->getValorlinp()->findValorlinp($clientId, $cdlinpclie, $initialDate);
        if(empty($valorLinp)) {
            $linhaProdutoFactory->insertValorlinp($requesterOrganizationId, $requesterUserId,
                $clientId, $cdlinpclie, $initialDate, $expirationDate, $vrlicencusolinp, $vrmensalidalinp);
        } else {
            $linhaProdutoFactory->updateValorlinp($requesterOrganizationId, $requesterUserId,
                $clientId, $cdlinpclie, $initialDate, $expirationDate, $vrlicencusolinp, $vrmensalidalinp);
        }

        foreach ($elements as $element) {
            /** @var Entities\Sistcontlinp $systemProductLine */
            $systemProductLine = InstanceProvider::getRepository()->getSistcontlinp()->findOneBy(
                array('cdsistema' => $element['CDSISTEMA'], 'cdcliente' => $clientId, 'cdlinpclie' => $cdlinpclie));
            if (empty($systemProductLine)) {
                $systemProductLine = $linhaProdutoFactory->insertSistContLinp($dateNow, $requesterOrganizationId,
                    $requesterUserId, $clientId, $cdlinpclie, $element['CDSISTEMA']);
                $this->entityManager->persist($systemProductLine);
            }
        }
    }

    private function saveElementsToOrganization($requesterOrganizationId, $requesterUserId, $productElements, $productOrg,
                                                $productDetails, $expirationDate, $isExtendContract, $contractCode)
    {
        foreach ($productElements as $element) {
            $elementEntity = $this->productElementRepository->findOneBy(array('uniqueId' => $element['id'], 'nrprodutoId' => $productOrg->getNrprodutoId()));
            if ($elementEntity) {
                $isUpdate = isset($element['isUpdate']) ? $element['isUpdate'] : true;
                $this->orgElementService->orgElementSaveIntegration($requesterOrganizationId, $requesterUserId,
                    $elementEntity->getId(), $productOrg->getId(), $expirationDate, $isExtendContract, $contractCode, $isUpdate);
            }
        }
        if(!$isExtendContract) {
            $elementIds       = Utilities::getArrayFromAssociativeArray($productElements, 'id');
            $elementIds       = empty($elementIds) ? array(0) : $elementIds;
            $elementsToDelete = InstanceProvider::getRepository()->getOmProdutoOrg()->findProductElementsToDeleteContract($productOrg->getNrprodutoId(), 
                $productOrg->getId(), $elementIds);
            foreach ($elementsToDelete as $element) {
                $this->orgElementService->setChildrenElementInvisible($productOrg->getId(), $element['ID'], $element['PARENT_ID']);
            }
        }
    }

    private function saveProductToOrganization($organizationId, $productId, $requesterUserId, $requesterOrganizationId, $dateNow)
    {
        try {
            $organizationProduct = $this->produtoOrgRepository->findOneBy(array('nrorgId' => $organizationId, 'nrprodutoId' => $productId));
            return $this->produtoOrgFactory->save($organizationProduct, $organizationId, $productId, $requesterUserId, $requesterOrganizationId, $dateNow);
        } catch (\Exception $e) {
            throw new \Exception(Messages::ERR_NOT_SAVE_ORGANIZATION_BECAUSE_OCCUR_UNKNOWN_ERROR_SAVE_PRODUCT);
        }
    }
    public function synchronizeElementsByProductAndCreate($productId, $code = null){
        $elementoService = InstanceProvider::getService()->getElemento();
        /** @var Entities\OmProduto $productDetails */
        $productDetails = $this->productRepository->findOneBy(array('id' => $productId));
        $nrexternal = $productDetails->getNrexternalProductId();
        $elemento = array(
            "LABEL"=> $productDetails->getNmproduto(),
            "NAME"=> "UNIQUE_NAME",
            "EX_PRODUCT_ID"=> $nrexternal,
            "PARENT_ID"=> null,
            "TYPE"=> "M",
            "NRORDEM"=> "1",
            "PROJECT_ID"=> $productId,
            "UNIQUE_ID"=> $nrexternal.'00000'.$productId
        );
        
        $elementoService->saveElements(array($elemento));
        return $this->synchronizeElementsByProduct($productId, $code);
    }

    public function synchronizeElementsByProduct($productId, $code = null)
    {
        $privateKey = Utilities::SECRET_CLIENTE_LIBRARIES;
        $requesterUrl = $this->utilities->getSmUrl();
        $token = $this->utilities->getLibToken($requesterUrl);
        if($token) {
            $productAndYourElements = $this->getProductAndYourElements($productId);
            array_push($productAndYourElements, array("code" => $code));
            $dataSet = array(
                "requestType" => "DataSet",
                "dataset" => array(
                    $productAndYourElements

                )
            );
            $this->utilities->chamaWebService($token, $this->utilities->getSmIntegrationRequestType(), $dataSet, 'synchronize', $privateKey, $requesterUrl);
            return array(true);
        } else {
            throw new \Exception(Messages::ERR_URL_UNDEFINED);
        }
    }

    private function getProductAndYourElements($productId)
    {
        $responseElements = array();
        /** @var Entities\OmProduto $productDetails */
        $productDetails = $this->productRepository->findOneBy(array('id' => $productId));
        if ($productDetails) {
            $elementsToSort = $this->productElementRepository->getElements($productId);
            $responseProduct = array(
                "id" => $productDetails->getId(),
                "idProdutoSaas" => $productDetails->getIdprodutosaas(),
                "name" => $productDetails->getNmproduto()
            );

            $elements = Utilities::generateMenuElementsTreeGrid($elementsToSort);

            if (count($elements) > 0) {
                foreach ($elements as $element) {
                    $parentUniqueId = null;
                    if (!empty($element["PARENT_ID"])) {
                        $parentUniqueId = $this->productElementRepository->findOneBy(array('nrprodutoId' => $productId, 'id' => $element["PARENT_ID"]))->getUniqueId();
                    }

                    array_push($responseElements, array(
                        "id" => $element["UNIQUE_ID"],
                        "name" => $element["LABEL"],
                        "parent" => $parentUniqueId
                    ));
                }
                return array(
                    "product" => $responseProduct,
                    "elements" => $responseElements,
                    "requester" => array(
                        "userId" => $this->environment->getCurrentUserId(),
                        "organizationId" => $this->environment->getCurrentOrganizationId()
                    )
                );
            } else {
                throw new \Exception(Messages::ERR_NOT_SYNCHRONIZE_PRODUCT_WITHOUT_ELEMENTS);
            }
        } else {
            throw new \Exception(Messages::ERR_NOT_SYNCHRONIZE_PRODUCT_NOT_FOUND);
        }
    }

    public function getElementsByDevOpsIdWithOrganizationFilter($productId, $cdcliente)
    {
        return InstanceProvider::getRepository()->getOmProdutoElemento()->getProductElementsByProductIdAndCdcliente($productId, $cdcliente);
    }

    public function sendCollectionToSM($row)
    {
        $privateKey = Utilities::SECRET_CLIENTE_LIBRARIES;
        $requesterUrl = $this->utilities->getSmUrl();
        $token = $this->utilities->getLibToken($requesterUrl);
        $dataSet = $this->getDataSetToSendCollection($row);
        $result = $this->utilities->chamaWebService($token, '/fct_medicaofatura/atualizarMedVariavelOM', $dataSet,
            'SUCCESS', $privateKey, $requesterUrl);

        if (empty($result)) {
            throw new \Exception(Messages::ERR_SEND_COLLECTION_TO_SM);
        }

        return array(true);
    }

    /**
     *
     * @throws \Exception
     */
    public function sendCollectionToSMByScript()
    {
        $productService = InstanceProvider::getService()->getProduto();
        echo "\n------------------------------------------------- Início -------------------------------------------------\n";
        echo "\n \n";
        echo ">>> Buscando produtos com medidas variáveis...\n\n";
        $products = InstanceProvider::getRepository()->getOmConfigMedidaVariavel()->findPendentContractsSendVariableMeasure();
        
        foreach ($products as $product) {
            echo ">>> Coletando medidas variáveis...\n\n";
            if(empty($product['NMPRODUTO'])) {
                echo "### Serviço: {$product['NMSERVORCA']}\n";
            } else {
                echo "### Produto: {$product['NMPRODUTO']}\n";
            }
            echo "### Organização: {$product['NMORGANIZACAO']}\n";

            $conditions = $this->getConditionsCollectionVariableMeasures($product);
            $variableMeasures = $productService->findCollectionVariableMeasures($conditions, '', 1, 500, array(), $product['NRPRODUTO_ID'], $product['NRORG']);

            if (count($variableMeasures)) {
                echo "### Número de medidas variáveis coletadas: " . count($variableMeasures) . ".\n\n";
            } else {
                echo "### Não foram coletadas medidas veriáveis.\n\n";
            }
            foreach ($variableMeasures as $row) {
                try {
                    echo ">>> Enviando dados para o SM...\n\n";
                    echo "### Id do contrato: {$row['contractId']} ({$row['initCompetence']} - {$row['endCompetence']})\n";
                    echo "### Data da coleta: {$row['date']}\n";
                    echo "### Quantidade: {$row['count']}\n\n";
                    $this->sendCollectionToSM($row);
                    echo "<<< Dados enviados com sucesso.\n\n";
                } catch (\Exception $e) {
                    if(strpos($e->getMessage(), self::ERROR_MSG_DISCARDED) === false){
                        $this->sendErrorEmail($product['NRORG'], $product['NRPRODUTO_ID'], $product['NMPRODUTO'], $product['NRSEQCONTRATO'], $row['measureName'], $row['initCompetence'], $row['endCompetence'], $e->getMessage());
                    }
                    echo "<<< Falha ao enviar dados.\n";
                    echo "<<< ERROR: ";
                    echo $e->getMessage() . "\n\n";
                }
                echo ">>> --------------------------------------- Fim do envio --------------------------------------------- <<<\n\n";
            }
        }
    }

    private function getConditionsCollectionVariableMeasures(array $product)
    {
        $conditions = [
            [
                'name' => 'contractId',
                'operator' => '=',
                'value' => strval($product['NRSEQCONTRATO'])
            ], [
                'name' => 'lastStatus',
                'operator' => '=',
                'value' => 'S'
            ], [
                'name' => 'initCompetence',
                'operator' => '=',
                'value' => $product['DTINICOMPET']
            ], [
                'name' => 'endCompetence',
                'operator' => '=',
                'value' => $product['DTFINCOMPET']
            ]
        ];
        if($product['CDSERVORCA']) {
            $conditions[] =  [
                'name' => 'serviceId',
                'operator' => '=',
                'value' => strval($product['CDSERVORCA'])
            ];
        }
        return $conditions;
    }

    private function getDataSetToSendCollection($row)
    {
        $product = $this->productRepository->findOneBy(array('nrexternalProductId' => $row['productId']));
        return array(
            "requestType" => "DataSet",
            "dataset" => array(array(
                "NRSEQCONTRATO"   => $row['contractId'],
                "NRMEDICAOFATURA" => $row['contractExpId'],
                "CDSERVORCA"      => $row['serviceId'],
                "NRORG"           => $this->environment->getCurrentOrganizationId() ?: Utilities::getTeknisaNrorgParameter(),
                "DTINICOMPET"     => is_string($row['initCompetence']) ? $row['initCompetence'] : \DateTime::createFromFormat('Ymd', $row['initCompetence'])->format('d/m/Y'),
                "DTFINCOMPET"     => is_string($row['endCompetence'])  ? $row['endCompetence']  : \DateTime::createFromFormat('Ymd', $row['endCompetence']) ->format('d/m/Y'),
                "DTCOLETA"        => is_string($row['date'])           ? $row['date']           : \DateTime::createFromFormat('Ymd', $row['date'])          ->format('d/m/Y'),
                "MEDVARIAVEIS"    => array(
                    array(
                        "CDTPMEDIDAVARIAVEL" => $row['measureCode'],
                        "QTMEDIDAVARIAVEL"   => $row['count'],
                        "NRPRODUTO"          => empty($product) ? null : $product->getId()
                    )
                )
            ))
        );
    }

    /**
     * @param $organizationId
     * @param $productId
     * @param $productName
     * @param $contractId
     * @param $measureName
     * @param $initCompetence
     * @param $endCompetence
     * @param $error
     * @throws \Exception
     */
    private function sendErrorEmail($organizationId, $productId, $productName, $contractId, $measureName, $initCompetence, $endCompetence, $error)
    {
        $emails = Utilities::getVariableMeasureEmailToSendParameter();
        foreach (explode(';', $emails) as $email) {
            $body = "<h3>Ocorreu um erro na rotina de enviar medidas variáveis para o SM. Dados abaixo:</h3>
                 <strong><li>Número da Organização:</strong> $organizationId</li>
                 <strong><li>ID do Produto:</strong> $productId</li>
                 <strong><li>Nome do Produto:</strong> $productName</li>
                 <strong><li>Número do contrato:</strong> $contractId</li>
                 <strong><li>Medida Variável:</strong> $measureName</li>
                 <strong><li>Período de competência:</strong> de $initCompetence até $endCompetence</li><br/>
                 <strong><li>Erro:</strong> $error</li>";

            $body = utf8_decode($body);
            $smtpMail = new SmtpCore();
            $smtpMail->enviaHtml($email, 'pm@zeedhi.com', self::VARIABLE_MEASURE_BASE_SUBJECT_EMAIL, $body);
        }
    }

    /**
     * @param $productId
     * @param $conditions
     * @return mixed
     */
    public function findOrganizationInformation($productId, $conditions)
    {
        $filters = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getOrganizacao()->findOrganizationInformation($productId, $filters);
    }

    /**
     * @param $productId
     * @param $serverId
     * @param $conditions
     * @return mixed
     * @throws \Exception
     */
    public function findServerMnemonic($productId, $serverId, $conditions)
    {
        $filters = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getServerMnemonic()->findServerMnemonicByProductServerIds($productId, $serverId, $filters);
    }

    /**
     * @param $productId
     * @param $productOrgId
     * @param $conditions
     * @return array
     * @throws \Exception
     */
    public function findProductElement($productId, $productOrgId, $conditions)
    {
        $filters = Utilities::createQueryFiltered($conditions);
        return InstanceProvider::getRepository()->getOmProdutoOrg()->findProductElement($productId, $productOrgId, $filters);
    }
}